#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pandas_ml_utils',
 'pandas_ml_utils.classification',
 'pandas_ml_utils.datafetching',
 'pandas_ml_utils.error',
 'pandas_ml_utils.extern',
 'pandas_ml_utils.model',
 'pandas_ml_utils.regression',
 'pandas_ml_utils.wrappers']

package_data = \
{'': ['*']}

install_requires = \
['pandas', 'cachetools', 'sortedcontainers', 'scikit-learn', 'dill']

extras_require = \
{'jupyther': ['vdom', 'matplotlib', 'seaborn'], 'stock_prices': ['yfinance']}

setup(name='pandas-ml-utils',
      version='0.0.7',
      description='Augment pandas DataFrame with methods for machine learning',
      author='KIC',
      author_email='kic@mindbusters.com',
      url='https://github.com/KIC/pandas_ml_utils',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>= 3.7',
     )
